/* native-ref.c: Native code of str-ref.
* This file is part of Transformers.
* Copyright (C) 2007  EPITA Research and Development Laboratory.
*
* Transformers is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
* USA.
*/
#include <srts/stratego.h>
#include "native-ref.h"

static
void
init_module_constant_terms(void)
{
}

static
void
init_module_constructors(void)
{
}

/**
 * This function is used to dereference a pointer include in the Ref
 * constructor.
 *
 * This function fail is 'r' is not a reference or if the
 * reference/value is invalid.
 */
ATerm
REF_deref (ATerm r)
{
  ATerm *v = NULL;

  if (ATmatch(r, "Ref(<int>)", &v) == ATfalse)
    _fail(r);

  if (v == NULL)
    _fail(r);

  return *v;
}

/**
 * This function is used to bind a reference.
 *
 * This function fail if 'r' is not a reference or if the pointer is
 * invalid.
 *
 * To manage this with the garbage collector of ATerm this function
 * use the ATunprotect and the ATprotect functions.
 */
ATerm
REF_bind (ATerm val, ATerm r)
{
  ATerm *v = NULL;

  if (ATmatch(r, "Ref(<int>)", &v) == ATfalse)
    _fail(r);

  if (v == NULL)
    _fail(r);

  ATunprotect(v);
  *v = val;
  ATprotect(v);

  return r;
}

/**
 * This function always succeeded except if there is not enough
 * memory. It return an unbound reference.
 */
ATerm
REF_create (void)
{
  ATerm *v = NULL;

  if ((v = malloc(sizeof (ATerm))) == NULL)
  {
    ATfprintf(stderr, "REF_create: allocation failed.\n");
    _fail(NULL);
  }

  *v = ATmake("Ref(<int>)", v);
  ATprotect(v);

  return *v;
}

/**
 * This function is used to remove unused reference.
 */
ATerm
REF_destroy (ATerm r)
{
  ATerm *v = NULL;

  if (ATmatch(r, "Ref(<int>)", &v) == ATfalse)
    _fail(r);

  if (v == NULL)
    _fail(r);

  ATunprotect(v);
  free(v);

  return ATmake("Ref(<int>)", NULL);
}
